
       subroutine formh 

c       subroutine formh(file)
C
C  Forms H center positions in a multichain bulk polypropylene model.
C
C  INPUT VARIABLES
C  ---------------
C  nchains    Number of chains in system.
C  nun        Degree of polymerization (number of units per chain).
C  ellH       C-H bond length (considered fixed), in the same units as h.
C  thetaH     Supplement of H-C-H bond angle for achiral segments, in degrees.
C  PDMSE doesnt need the above two lines!
C
C  x0,y0,z0   Arrays containing box-scaled coordinates of the box images of
C             primary atoms (methyls, methines, methylenes).
C             The coordinates of primary atom iprim of chain ichain are
C             contained in x0(iprim,ichain), y0(iprim,ichain), z0(iprim,ichain).
C             Primary atoms are assigned indices iprim = 1,2,...,3nun+1
C             in the following sequence:
C
C             C0,R1,C1,C2,R3,C3,...,C2nun-2,R2nun-1,C2nun-1,C2nun.
C
C             Explanation of sequence(PP):
C
C
C               R1         R3           R2n-3 R2n-1
C                .        -            -     -
C                 .      -            -     -
C                 C1    C3           C2n-3 C2n-1
C                / \   / \    . . . / \   / \           (with n = nun)
C               /   \ /   \ /          \ /   \
C              C0    C2    C4           C2n-2 C2n
C
C
C  ichir      Array descriptive of the chirality sequence of all chiral
C             carbons along the chains.
C             ichir(i,ichain) = +1 if carbon C2i-1 of chain ichain is
C                               D in a Fischer projection constructed with
C                               the chain start upwards.
C
C             ichir(i,ichain) = -1 if carbon C2i-1 of chain ichain is
C                               L in a Fischer projection constructed with
C                               the chain start upwards.
C
C             Note: Even the first and last skeletal carbons of the chain,
C                   primary atoms 3 and 3nun, are attributed a chirality,
C                   by convention.
C
C             Chirality sequence is computed from the parent image coordinates
C             of the primary atoms, when formh is called for the first
C             time (first = .TRUE.).
C  PDMSE doesnt need that stuff, either!
C
C  h          Matrix containing the three edge vectors of the box as
C             column vectors.  It is required to convert box-scaled
C             coordinates to real-space coordinates.  The determinant of
C             h is the box volume, vol. h must be provided upon entry.
C
C  hi         Inverse of matrix h.  hi and vol are computed within this
C             routine.
C
C  first      A logical flag.  Should be set to .TRUE. just before formh
C             uis called for the first time.
C
C  OUTPUT VARIABLES
C  ----------------
C
C  xf0,yf0,zf0
C             Arrays containing the box-scaled coordinates of box images of
C             _all atoms_, including hydrogens.
C             The coordinates of atom iatom of chain ichain are contained
C             in xf0(iatom,ichain), yf0(iatom,ichain), zf0(iatom,ichain).
C             Atoms are assigned the indices iatom=1,2,...,6nun-1
C             according to the sequence:
C
C             C0,R1,H1,C1,H2-,H2+,C2,R3,H3,C3,H4-,H4+,C4,...,
C                H2nun-2-,H2nun-2+,C2nun-2,R2nun-1,H2nun-1,C2nun-1,C2nun
C
C             with + and - signifying hydrogen atoms of achiral centers that
C             occupy a D and L position, respectively, in a Fischer projection
C             of the chain constructed with the start at the top.
C
C  As regards PDMSE the following sequence applies:
C
C	      C0,R1,R2,C1,C2,R3,R4,C3,C4,R5,R6,C5,...
C
C  More explicitly, the atom numbering scheme is as follows:
C
C  Heavy Atom Representation (x0,y0,z0)
C  -------------------------
C   Terminal methyls: 1, 3nun+1
C   R-substituents:   2, 5, ..., 3nun-1. Generally, 3k-1 with k=1,2,...,nun
C   Chiral carbons:   3, 6, ..., 3nun  . Generally, 3k   with k=1,2,...,nun
C   Achiral carbons:  4, 7, ..., 3nun-2. Generally, 3k+1 with k=1,2,...,nun-1
C
C  All-Atom Representation (xf0,yf0,zf0)
C  -----------------------
C   Terminal methyls: 1, 6nun-1
C   R-substituents:   2, 8,16,...,6nun-4. Generally, 6k-4 with k=1,2,...,nun
C   Chiral H-subst.:  3, 9,15,...,6nun-3. Generally, 6k-3 with k=1,2,...,nun
C   Chiral carbons:   4,10,16,...,6nun-2. Generally, 6k-2 with k=1,2,...,nun
C   Achiral H-(L)sub: 5,11,17,...,6nun-7. Generally, 6k-1 with k=1,2,...,nun-1
C   Achiral H+(D)sub: 6,12,18,...,6nun-6. Generally, 6k   with k=1,2,...,nun-1
C   Achiral carbons:  7,13,19,...,6nun-5. Generally, 6k+1 with k=1,2,...,nun-1
C
C The above arrays coincide in the case of PDMSE!
C
C   rx,ry,rz  Arrays containing the real-space coordinates of the box images
C             of all primary atoms, i.e. the real-space counterparts of
C             x0,y0,z0.  Indexing is as in x0,y0,z0.
C
C   rfx,rfy,rfz
C             Arrays containing the real-space coordinates of the box images
C             of _all_ atoms, including hydrogens.  These are the real-space
C             counterparts of xf0, yf0, zf0.  Indexing as in xf0, yf0, zf0.
C
C   rpx,rpy,rpz
C             Arrays containing the real-space coordinates of the parent
C             chain images of all primary atoms.  Indexing as in x0,y0,z0.
C
C   rfpx,rfpy,rfpz
C             Arrays containing the real-space coordinates of the parent
C             chain images of _all_ atoms, including hydrogens.  Indexing
C             as in xf0, yf0, zf0.
C
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

#include "ppmdparam1.h"
#include "ppmdparam2.h"

      integer ich,iprim,jj,ichain
      integer i21,i2,i3,i6,ic,i,j,k,nu
      integer i31,i32,i3m3, i2m1, i3p1, ip1
      integer nx, ny, nz
      integer natpm1,natpm2
C     integer ll
C

      real*8 vec1(3), vec2(3),drmij(3)
      real*8 term1(3), term2(3), ccoef1, ccoef2
      real*8 rminsq, rpminsq,drm12(3), rmin
      real*8 x, y, z,w,pi,degrad
      real*8 tmp
C
c      logical first

#include "bonds.h"
#include "box.h"
#include "coeff.h"
#include "geom.h"
#include "parents.h"
#include "posit.h"
#include "scalpos.h"
#include "sizes.h"
#include "which.h"

c #include "pentrnt.h"
c #include "bonds_pen.h"
#include "steps.h"
#include "times.h"
#include "listi.h"
c #include "listpen.h"
	logical condition
C steps.h and condition have been included for DEBUGGING
C times.h has been included for DEBUGGING ON AEGEAN 091200-1

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
C  Form and store the 27 possible continuation vectors in box-scaled form
C  (xl,yl,zl) and in real-space form (rxl,ryl,rzl)
      do i=1,3
          do j=1,3
              do k=1,3
                  nu=9*i+3*j+k-12
                  xl(nu)=dfloat(i-2)
                  yl(nu)=dfloat(j-2)
                  zl(nu)=dfloat(k-2)
              end do
          end do
      end do
      do nu =1,27
	  rxl(nu) = h(1,1)*xl(nu) + h(1,2)*yl(nu) + h(1,3)*zl(nu)
	  ryl(nu) = h(2,1)*xl(nu) + h(2,2)*yl(nu) + h(2,3)*zl(nu)
	  rzl(nu) = h(3,1)*xl(nu) + h(3,2)*yl(nu) + h(3,3)*zl(nu)
      end do
C
C
C  First make sure that all box-scaled images of primary atoms, stored
C  in x0, y0, z0, lie inside the primary box.  (Some may have exited
C  in the course of the MD integration).
C
      do ichain = 1, nchains
          do iprim = 1, natmc(ichain)
              x = x0mc(iprim,ichain)
              y = y0mc(iprim,ichain)
              z = z0mc(iprim,ichain)
C
              nx = x + x
              x0mc(iprim,ichain) = x - nx
              ny = y + y
              y0mc(iprim,ichain) = y - ny
              nz = z + z
              z0mc(iprim,ichain) = z - nz

	if ((abs(x0mc(iprim,ichain)).gt.0.5d0)
     &	.or.(abs(y0mc(iprim,ichain)).gt.0.5d0)
     &  .or.(abs(z0mc(iprim,ichain)).gt.0.5d0)) then
	  write(6,*) 'nstep=',nstep,'images out of the box'
	  write(6,*) 'elapse=',elapse
	  write(6,*) 'atom=',iprim,'chain=',ichain
	  write(6,444) x,y,z
	  write(6,444) x0mc(iprim,ichain),y0mc(iprim,ichain),
     &                 z0mc(iprim,ichain)
          STOP "formh: images out of the box"
	endif 
          enddo
          do iprim = 1, natsu(ichain)
              x = x0su(iprim,ichain)
              y = y0su(iprim,ichain)
              z = z0su(iprim,ichain)
C
              nx = x + x
              x0su(iprim,ichain) = x - nx
              ny = y + y
              y0su(iprim,ichain) = y - ny
              nz = z + z
              z0su(iprim,ichain) = z - nz

	if ((abs(x0su(iprim,ichain)).gt.0.5d0)
     &	.or.(abs(y0su(iprim,ichain)).gt.0.5d0)
     &  .or.(abs(z0su(iprim,ichain)).gt.0.5d0)) then
	  write(6,*) 'nstep=',nstep,'images out of the box'
	  write(6,*) 'elapse=',elapse
	  write(6,*) 'atom=',iprim,'chain=',ichain
	  write(6,444) x,y,z
	  write(6,444) x0su(iprim,ichain),y0su(iprim,ichain),
     &                 z0su(iprim,ichain)
          STOP "formh: images out of the box"
	endif 
          enddo
      enddo
  444 format(t1,f8.3,t12,f8.3,t23,f8.3)
C
C
C  Compute real-space coordinates of box images of primary atoms,
C  and place them in rx, ry, rz
      do ichain = 1, nchains
	  do iprim = 1,natmc(ichain)
	      rxmc(iprim,ichain) = h(1,1) * x0mc(iprim,ichain) +
     1                           h(1,2) * y0mc(iprim,ichain) +
     2                           h(1,3) * z0mc(iprim,ichain)
	      rymc(iprim,ichain) = h(2,1) * x0mc(iprim,ichain) +
     1                           h(2,2) * y0mc(iprim,ichain) +
     2                           h(2,3) * z0mc(iprim,ichain)
	      rzmc(iprim,ichain) = h(3,1) * x0mc(iprim,ichain) +
     1                           h(3,2) * y0mc(iprim,ichain) +
     2                           h(3,3) * z0mc(iprim,ichain)
	  enddo
	  do iprim = 1,natsu(ichain)
	      rxsu(iprim,ichain) = h(1,1) * x0su(iprim,ichain) +
     1                           h(1,2) * y0su(iprim,ichain) +
     2                           h(1,3) * z0su(iprim,ichain)
	      rysu(iprim,ichain) = h(2,1) * x0su(iprim,ichain) +
     1                           h(2,2) * y0su(iprim,ichain) +
     2                           h(2,3) * z0su(iprim,ichain)
	      rzsu(iprim,ichain) = h(3,1) * x0su(iprim,ichain) +
     1                           h(3,2) * y0su(iprim,ichain) +
     2                           h(3,3) * z0su(iprim,ichain)
	  enddo
      enddo
C
C
C  Form and store chain bond vectors and parent image coordinates of
C  heavy atoms.
C
C                                                       ^
C  In array b are stored the skeletal bond unit vectors b  of all chains.
C                                                       ~j
C  in the direction chain start to chain end.
C
C
C  In array blen are stored the lengths of all skeletal bonds, l
C                                                               j
C          (j = 1,2,...,2nun)
C
C                                             ^    ^
C  In array bxb are stored the cross products b  x b
C                                             ~j   ~j+1
C
C                                            ^    ^
C  In array IbxbI are stored the values of | b  x b   |
C                                            ~j   ~j+1
C
C                                             ^   ^
C  In array bdotb are stored the dot products b . b
C                                             ~j  ~j+1
C
C                                               ^      ^
C  In array bbdotb are stored the dot products  b   .  b
C                                               ~j-1   ~j+1
C
C                                                 ^    ^    -1/2
C  In array ri1mbb are stored the quantities (1 - b  . b   )
C                                                 ~j   ~j+1
C
C                                                 ^    ^    -1/2
C  In array ri1pbb are stored the quantities (1 + b  . b   )
C                                                 ~j   ~j+1
C
C                                                          ^
C  In array d are stored the pendant C-R bond unit vectors d
C                                                           2j-1
C
C  of all chains (in the direction from skeletal carbon to R substituent).
C
C  In array dlen are stored the lengths of all pendant bonds 2j-1
C     (j = 1, 2, ..., nun)
C
      do ich = 1, nchains
C
          do i=1,natmc(ich)-1

c       Form main chain bond

            call minimg(i,ich,1,i+1,ich,1,drmij,rminsq,rpminsq)
            rmin=dsqrt(rminsq)
            blen(i,ich)=rmin
            b(1,i,ich)=drmij(1)
            b(2,i,ich)=drmij(2)
            b(3,i,ich)=drmij(3)
          enddo
C
C
C    .... At this point, b contains the actual real-space skeletal bond vectors
C    .... (not unit bond vectors).  Use those to form parent chain images of
C    .... skeletal atoms.
C    .... NOTE:  This is a highly sequential calculation.

          rpxmc(1,ich)=rxmc(1,ich)
          rpymc(1,ich)=rymc(1,ich)
          rpzmc(1,ich)=rzmc(1,ich)

          do i=2,natmc(ich)
            rpxmc(i,ich)=rpxmc(i-1,ich)+b(1,i-1,ich)
            rpymc(i,ich)=rpymc(i-1,ich)+b(2,i-1,ich)
            rpzmc(i,ich)=rpzmc(i-1,ich)+b(3,i-1,ich)
           end do
C
C    .... Now convert all bond vectors b to unit vectors.
          do i = 1, natmc(ich)-1
              rmin = blen(i,ich)
              b(1,i,ich) = b(1,i,ich)/rmin
              b(2,i,ich) = b(2,i,ich)/rmin
              b(3,i,ich) = b(3,i,ich)/rmin
          enddo
C
C    .... Form bxb, bdotb, ri1mbb, ri1pbb
c              (bbdotb no longer formed)

          bxb(1,1,ich)=+b(2,1,ich)*b(3,2,ich)
     &                 -b(3,1,ich)*b(2,2,ich)
          bxb(2,1,ich)=+b(3,1,ich)*b(1,2,ich)
     &                 -b(1,1,ich)*b(3,2,ich)
          bxb(3,1,ich)=+b(1,1,ich)*b(2,2,ich)
     &                 -b(2,1,ich)*b(1,2,ich)
	  bdotb(1,ich) = b(1,1,ich)*b(1,2,ich) +
     1                   b(2,1,ich)*b(2,2,ich) +
     2                   b(3,1,ich)*b(3,2,ich)
          IbxbI(1,ich)=dsqrt(bxb(1,1,ich)*bxb(1,1,ich)
     &                        +bxb(2,1,ich)*bxb(2,1,ich)
     &                        +bxb(3,1,ich)*bxb(3,1,ich))

	      ri1mbb(1,ich) = 1.D+00/sqrt(1.0D+00 - bdotb(1,ich))
	      ri1pbb(1,ich) = 1.D+00/sqrt(1.0D+00 + bdotb(1,ich))
              tmp=1.D+00/(sqrt(1.D+00 - bdotb(1,ich)**2))
	      vvect(1,1,ich) = -bxb(1,1,ich)*tmp
	      vvect(2,1,ich) = -bxb(2,1,ich)*tmp
	      vvect(3,1,ich) = -bxb(3,1,ich)*tmp


          do  i=2,natmc(ich)-2

            bxb(1,i,ich)=+b(2,i,ich)*b(3,i+1,ich)
     &                   -b(3,i,ich)*b(2,i+1,ich)
            bxb(2,i,ich)=+b(3,i,ich)*b(1,i+1,ich)
     &                   -b(1,i,ich)*b(3,i+1,ich)
            bxb(3,i,ich)=+b(1,i,ich)*b(2,i+1,ich)
     &                   -b(2,i,ich)*b(1,i+1,ich)
	    bdotb(i,ich) = b(1,i,ich)*b(1,i+1,ich) +
     1                     b(2,i,ich)*b(2,i+1,ich) +
     2                     b(3,i,ich)*b(3,i+1,ich)
            IbxbI(i,ich)=dsqrt(bxb(1,i,ich)*bxb(1,i,ich)
     &                        +bxb(2,i,ich)*bxb(2,i,ich)
     &                        +bxb(3,i,ich)*bxb(3,i,ich))

	      ri1mbb(i,ich) = 1.D+00/sqrt(1.0D+00 - bdotb(i,ich))
	      ri1pbb(i,ich) = 1.D+00/sqrt(1.0D+00 + bdotb(i,ich))
              tmp=1.D+00/(sqrt(1.D+00 - bdotb(i,ich)**2))
	      vvect(1,i,ich) = -bxb(1,i,ich)*tmp
	      vvect(2,i,ich) = -bxb(2,i,ich)*tmp
	      vvect(3,i,ich) = -bxb(3,i,ich)*tmp
	enddo

C
C    .... Pendant C-R bonds: Pendant - Skeletal

          do i=1,natsu(ich)

            call minimg(sutmc(i),ich,1,i,ich,2,drmij,rminsq,rpminsq)
            rmin=dsqrt(rminsq)
            dlen(i,ich)=rmin
            d(1,i,ich)=drmij(1)
            d(2,i,ich)=drmij(2)
            d(3,i,ich)=drmij(3)
          enddo
C
C    .... At this point, d contains the actual real-space C-R bond vectors
C    .... (not unit vectors).  Use those to form parent chain images of
C    .... R substituents.

          do  i=1,natsu(ich)
            rpxsu(i,ich)=rpxmc(sutmc(i),ich)+d(1,i,ich)
            rpysu(i,ich)=rpymc(sutmc(i),ich)+d(2,i,ich)
            rpzsu(i,ich)=rpzmc(sutmc(i),ich)+d(3,i,ich)

C    .... Convert all d vectors to unit vectors.
              
			rmin = dlen(i,ich)
              d(1,i,ich) = d(1,i,ich)/rmin
              d(2,i,ich) = d(2,i,ich)/rmin
              d(3,i,ich) = d(3,i,ich)/rmin
c	write(111,*) nstep,d(1,i,ich)**2+d(2,i,ich)**2+d(3,i,ich)**2
          enddo

      enddo
C
#ifdef NOTHING

	do j=1,nchains
             do k=1,natmc(j)+natsu(j)-1
                  i=listbli(k)
                  jj=listblj(k)
                  x=x0(i,j)-x0(jj,j)
                  y=y0(i,j)-y0(jj,j)
                  z=z0(i,j)-z0(jj,j)
                  nx=x+x
                  ny=y+y
                  nz=z+z
                  mibl(k,j)=9*nx+3*ny+nz+14
             end do
	enddo


#endif
      return
      end
